//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "DModU.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDM *DM;

const int cnFilterDiff = 2;  //  cssContactType0ʼӦFilterType2
//---------------------------------------------------------------------------

__fastcall TDM::TDM(TComponent* Owner)
	: TDataModule(Owner),
	zConn( new TZConnection( this ) ),
	qryData( new TZQuery( this ) ),
	tblData( new TZTable( this ) )
{
	zConn->Protocol = "sqlite-3";
	zConn->HostName = "localhost";
	zConn->Database = ChangeFileExt( Application->ExeName, ".dat" );
	zConn->User     = "";
	zConn->Password = "";
	zConn->Connect();
	qryData->Connection = zConn.get();
	tblData->Connection = zConn.get();
}
//---------------------------------------------------------------------------
TZQuery * __fastcall TDM::GetTreeData( TFilterType aFilterType, AnsiString aFilter )
{
	qryData->Close( );
    switch ( aFilterType ) {
    case ftByName:
        qryData->SQL->Text = "SELECT g.gid, g.title, p.pid, p.name FROM pgroup g \
        LEFT JOIN person p ON g.gid = p.gid WHERE p.name LIKE :P_NAME \
        ORDER BY g.title, p.name";
        qryData->ParamByName( "P_NAME" )->AsString = AnsiString( "%" ) + aFilter + "%";
    	break;
    case ftByPhone:
    case ftByEmail:
    case ftByAddress:
        qryData->SQL->Text = "SELECT DISTINCT g.gid, g.title, p.pid, p.name FROM pgroup g \
        LEFT JOIN person p ON g.gid = p.gid INNER JOIN contact c ON c.pid = p.pid \
        WHERE c.ctid = :P_CTYPE AND c.contact LIKE :P_CONTACT \
        ORDER BY g.title, p.name";
        qryData->ParamByName( "P_CTYPE" )->AsInteger = aFilterType - cnFilterDiff;
        qryData->ParamByName( "P_CONTACT" )->AsString = AnsiString( "%" ) + aFilter + "%";
    	break;
    default:  // ftAll
        qryData->SQL->Text = "SELECT g.gid, g.title, p.pid, p.name FROM pgroup g \
		LEFT JOIN person p ON g.gid = p.gid ORDER BY g.title, p.name";
    }
	qryData->Open( );
	return qryData.get();
}
//---------------------------------------------------------------------------

TZQuery * __fastcall TDM::GetPersonData( int aPID, AnsiString &aDesc )
{
    qryData->Close( );
    qryData->SQL->Text = "SELECT desc FROM person WHERE pid = :P_PID";
    qryData->ParamByName( "P_PID" )->AsInteger = aPID;
    qryData->Open( );
    aDesc = qryData->FieldByName( "desc" )->AsString;
    qryData->Close( );
    qryData->SQL->Text = "SELECT c.cid, c.validate, c.ctid, cc.typename, c.contact \
    FROM contact c INNER JOIN ccategory cc ON c.ccid = cc.ccid \
    WHERE c.pid = :P_PID";
    qryData->ParamByName( "P_PID" )->AsInteger = aPID;
    qryData->Open( );
    return qryData.get();
}
//---------------------------------------------------------------------------

TZQuery * __fastcall TDM::GetCategories( )
{
    qryData->Close( );
    qryData->SQL->Text = "SELECT ccid, typename FROM ccategory";
    qryData->Open( );
    return qryData.get();
}
//---------------------------------------------------------------------------

int __fastcall TDM::AddGroup( AnsiString aGroupName )
{
	int gid = 1;
	qryData->Close( );
    qryData->SQL->Text = "SELECT MAX(gid) AS gid FROM pgroup";
    qryData->Open( );
    if ( !qryData->FieldByName( "gid" )->IsNull ) {
        gid = qryData->FieldByName( "gid" )->AsInteger + 1;
    }
    qryData->Close( );
    tblData->Close( );
	tblData->TableName = "pgroup";
    tblData->Open( );
    tblData->Append( );
    tblData->FieldByName( "gid" )->AsInteger = gid;
    tblData->FieldByName( "title" )->AsString = aGroupName;
    tblData->Post( );
    tblData->Close( );
    return gid;
}
//---------------------------------------------------------------------------

void __fastcall TDM::EditGroup( int aGID, AnsiString aGroupName )
{
    tblData->Close( );
	tblData->TableName = "pgroup";
    tblData->Open( );
    Variant vGID[1];
    vGID[0] = aGID;
    if ( tblData->Locate( "gid", VarArrayOf( vGID, 0 ), TLocateOptions( ) ) )
    {
        tblData->Edit( );
        tblData->FieldByName( "title" )->AsString = aGroupName;
        tblData->Post( );
    }
    tblData->Close( );
}
//---------------------------------------------------------------------------

void __fastcall TDM::DelGroup( int aGID )
{
	tblData->TableName = "pgroup";
    tblData->Open();
    Variant vGID[1];
    vGID[0] = aGID;
    if ( tblData->Locate( "gid", VarArrayOf( vGID, 0 ), TLocateOptions( ) ) )
        tblData->Delete();
    tblData->Close();
}
//---------------------------------------------------------------------------

int __fastcall TDM::AddPerson( int aGID, AnsiString aName, AnsiString aDesc )
{
    int pid = 1;
	qryData->Close( );
    qryData->SQL->Text = "SELECT MAX(pid) AS pid FROM person";
    qryData->Open( );
    if ( !qryData->FieldByName( "pid" )->IsNull ) {
        pid = qryData->FieldByName( "pid" )->AsInteger + 1;
    }
    qryData->Close( );
    tblData->Close( );
	tblData->TableName = "person";
    tblData->Open( );
    tblData->Append( );
    tblData->FieldByName( "gid" )->AsInteger = aGID;
    tblData->FieldByName( "pid" )->AsInteger = pid;
    tblData->FieldByName( "name" )->AsString = aName;
    tblData->FieldByName( "desc" )->AsString = aDesc;
    tblData->FieldByName( "validate" )->AsDateTime = Now();
    tblData->Post( );
    tblData->Close( );
    return pid;
}
//---------------------------------------------------------------------------

void __fastcall TDM::EditPerson( int aPID, AnsiString aName, AnsiString aDesc,
    int aGID )
{
    tblData->Close( );
	tblData->TableName = "person";
    tblData->Open( );
    Variant vPID[1];
    vPID[0] = aPID;
    if ( tblData->Locate( "pid", VarArrayOf( vPID, 0 ), TLocateOptions( ) ) )
    {
        if ( aGID == -1 ) {
            aGID = tblData->FieldByName( "gid" )->AsInteger;
        }
        tblData->Edit( );
        tblData->FieldByName( "gid"  )->AsInteger = aGID;
        tblData->FieldByName( "name" )->AsString  = aName;
        tblData->FieldByName( "desc" )->AsString  = aDesc;
        tblData->Post( );
    }
    tblData->Close( );
}
//---------------------------------------------------------------------------

void __fastcall TDM::DelPerson( int aPID )
{
    tblData->Close( );
	tblData->TableName = "person";
    tblData->Open( );
    Variant vPID[1];
    vPID[0] = aPID;
    if ( tblData->Locate( "pid", VarArrayOf( vPID, 0 ), TLocateOptions( ) ) )
        tblData->Delete();
    tblData->Close( );
}
//---------------------------------------------------------------------------

int __fastcall TDM::AddContact( int aPID, int aCCID, int aCTID, AnsiString aContact )
{
    int cid = 1;
	qryData->Close( );
    qryData->SQL->Text = "SELECT MAX(cid) AS cid FROM contact";
    qryData->Open( );
    if ( !qryData->FieldByName( "cid" )->IsNull ) {
        cid = qryData->FieldByName( "cid" )->AsInteger + 1;
    }
    qryData->Close( );
    tblData->Close( );
	tblData->TableName = "contact";
    tblData->Open( );
    tblData->Append( );
    tblData->FieldByName( "cid" )->AsInteger = cid;
    tblData->FieldByName( "pid" )->AsInteger = aPID;
    tblData->FieldByName( "ccid" )->AsInteger = aCCID;
    tblData->FieldByName( "ctid" )->AsInteger = aCTID;
    tblData->FieldByName( "validate" )->AsDateTime = Now( );
    tblData->FieldByName( "contact" )->AsString = aContact;
    tblData->Post( );
    tblData->Close( );
    return cid;
}
//---------------------------------------------------------------------------

void __fastcall TDM::DelContact( int aCID )
{
    tblData->Close( );
	tblData->TableName = "contact";
    tblData->Open( );
    Variant vCID[1];
    vCID[0] = aCID;
    if ( tblData->Locate( "cid", VarArrayOf( vCID, 0 ), TLocateOptions( ) ) )
        tblData->Delete( );
    tblData->Close( );
}
//---------------------------------------------------------------------------

